#!BPY

"""
Name: 'Pahts Wizards'
Blender: 245
Group: 'Wizards'
Submenu: 'Remove extensions from texture and image records' i_extrem
Submenu: 'Add reserved sign to shell' i_reserved
Submenu: 'Create a shell file' i_create
Submenu: 'Add collider under city link points' i_addcol
Tooltip: 'Ahra Pahts Wizards'
"""

__author__ = "Robert The Rebuilder"
__url__ = ("blender", "elysiun",
"Author's homepage, http://www.guildofwriters.com")
__version__ = "1.0"

__bpydoc__ = """\
This script provides convenience scripts for Ahra Pahts management.
"""

import Blender, time, sys, os
from os.path import *
from alc_Functions import *
from alc_AlcScript import *

def uncompress_texture():

    PROPERTY_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Object Name: ', PROPERTY_STRING, 19, 19, 'Final part of object name'),\
	]
		
    if not Blender.Draw.PupBlock('Specify objects to change...', pup_block) or\
    not PROPERTY_STRING.val:
        return 0
		
    PROPERTY_STRING = PROPERTY_STRING.val
    changeCount = 0		
    Blender.Window.WaitCursor(1)

    scene = Blender.Scene.GetCurrent()
    objlist = scene.objects

    for obj in objlist:
        if obj.name.endswith(PROPERTY_STRING):
            changeCount += 1
            if obj.type == 'Mesh':
                mesh = Blender.Mesh.Get(obj.data.name)
                if len(mesh.materials) > 0:
                    mat = mesh.materials[0]
                    for tex in mat.getTextures():
                        if (tex != None):
                            tex.tex.imageFlags &= ~Blender.Texture.ImageFlags["INTERPOL"]

    Blender.Window.WaitCursor(0)
	
    Blender.Draw.PupMenu('changed %d objects.' % (changeCount))

def delete_objects():

    PROPERTY_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Object Name: ', PROPERTY_STRING, 19, 19, 'Initial part of object name'),\
	]
		
    if not Blender.Draw.PupBlock('Specify objects to delete...', pup_block) or\
    not PROPERTY_STRING.val:
        return 0
		
    PROPERTY_STRING = PROPERTY_STRING.val
    changeCount = 0		
    Blender.Window.WaitCursor(1)

    scene = Blender.Scene.GetCurrent()
    objlist = scene.objects
    deletelist = []
    for obj in objlist:
        if obj.name.startswith(PROPERTY_STRING):
            deletelist.append(obj)

    for obj in deletelist:
        scene.objects.unlink(obj)
        changeCount += 1

    Blender.Window.WaitCursor(0)
	
    Blender.Draw.PupMenu('changed %d objects.' % (changeCount))

def import_subdistrict_objects(filename):
    subdistrictObjects = [
      [["StWal.2663","StWal.2622","StWal.2621","StWal.2654"],
       ["StWal.2646","StWal.1880","StWal.1854","StWal.2662"],
       ["StWal.2647","StWal.2623","StWal.2239","StWal.2655"],
       []],
      [["Wal.026","Wal.028","Wal.014","Wal.015","Wal.030","Wal.029","StWal.291",
        "StWal.336","StWal.369","StWal.412"],
       ["StWal.2674","StWal.2650","StWal.301","StWal.289"],
       ["StWal.038","StWal.017","StWal.396","StWal.531"],
       ["StWal.117","StWal.530","StWal.2652","StWal.2671"]],
      [["Wal.008","Wal.007","Wal.031","Wal.496","Wal.498","Wal.497",
        "StWal.376","StWal.050","StWal.290","StWal.335"],
       ["StWal.2673","StWal.2658","StWal.393","StWal.528"],
       ["StWal.394","StWal.370","StWal.325","StWal.077"],
       ["StWal.2672","StWal.2659","StWal.331","StWal.340"]],
      [["Wal","Wal.017","Wal.013","Wal.002","Wal.001","Wal.016","StWal.409","StWal.377","StWal.320"],
       ["StWal.2668","StWal.2643","StWal.342","StWal.064"],
       ["StWal.288","StWal.332","StWal.300","StWal.461"],
       ["StWal.249","StWal.001","StWal.2642","StWal.2667"]]]

    subdistrictNumbers = [[151,152,153,0],
      [251,252,253,254],
      [351,352,353,354],
      [451,452,453,454]]

    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()


    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"

    for district in range(4):
        for subdistrict in range(4):
            pageNum = subdistrictNumbers[district][subdistrict]
            print "Objects for district",district,",subdistrict",subdistrict,",page",pageNum,":"
            for objectName in subdistrictObjects[district][subdistrict]:
                print "\t",objectName
                for obname in Blender.Library.Datablocks(dbname):
                    if obname == objectName:
                        Blender.Library.Load(obname,dbname,0)
                        obj = Blender.Object.Get(objectName)
                        try:
                            p=obj.getProperty("page_num")
                            obj.removeProperty(p)
                        except (AttributeError, RuntimeError):
                            pass
                        obj.addProperty("page_num",str(pageNum))

    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()


def import_subdistrict_objects2(filename):
    subdistrictObjects = [
      [["StWal.030","StWal.2664","StWal.2620","StWal.2653"],
       ["StWal.2661","StWal.1764","StWal.1790","StWal.2645"],
       ["StWal.2126","StWal.2648","StWal.029","StWal.2656"],
       []],
      [["StWal.238","StWal.105","StWal.072","StWal.027"],
       ["StWal.2649","StWal.037","StWal.262","StWal.2636"],
       ["StWal.038","StWal.016","StWal.222","StWal.207"],
       ["StWal.2652","StWal.2634","StWal.117","StWal.201"]],
      [["StWal.071","StWal.026","StWal.050","StWal.139"],
       ["StWal.219","StWal.206","StWal.2640","StWal.2657"],
       ["StWal.220","StWal.130","StWal.167","StWal.077"],
       ["StWal.166","StWal.076","StWal.2660","StWal.2639"]],
      [["StWal.056","StWal.145","StWal.114"],
       ["StWal.2628","StWal.2644","StWal.064","StWal.078"],
       ["StWal.169","StWal.079","StWal.036","StWal.024"],
       ["StWal.2625","StWal.028","StWal","StWal.249"]]]

    subdistrictNumbers = [[151,152,153,0],
      [251,252,253,254],
      [351,352,353,354],
      [451,452,453,454]]

    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()


    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"

    for district in range(4):
        for subdistrict in range(4):
            pageNum = subdistrictNumbers[district][subdistrict]
            print "Objects for district",district,",subdistrict",subdistrict,",page",pageNum,":"
            for objectName in subdistrictObjects[district][subdistrict]:
                print "\t",objectName
                for obname in Blender.Library.Datablocks(dbname):
                    if obname == objectName:
                        Blender.Library.Load(obname,dbname,0)
                        obj = Blender.Object.Get(objectName)
                        try:
                            p=obj.getProperty("page_num")
                            obj.removeProperty(p)
                        except (AttributeError, RuntimeError):
                            pass
                        obj.addProperty("page_num",str(pageNum))

    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

def import_shell_image_wrap(filename):
    for district in range(4):
        shellLimit = 40
        if district == 0:
            shellLimit = 30
        for shellNum in range(shellLimit):
            SHELL_STRING="%d%02d" %(district+1,shellNum+1)
            print "Adding shell image to shell",SHELL_STRING,"..."
            import_shell_image(filename,SHELL_STRING)

def import_shell_image(filename,SHELL_STRING):
    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()
    objectName = SHELL_STRING + "ShellImage"
    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"
    for obname in Blender.Library.Datablocks(dbname):
        if obname == objectName:
            Blender.Library.Load(obname,dbname,0)
    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

def import_portals(filename):
    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()
    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"
    Portals = [
        "Portal11To24",
        "Portal11To32",
        "Portal12To34",
        "Portal13To22",
        "Portal13To44",
        "Portal21To22",
        "Portal21To23",
        "Portal21To24",
        "Portal22To13",
        "Portal22To21",
        "Portal23To21",
        "Portal24To11",
        "Portal24To21",
        "Portal31To32",
        "Portal31To33",
        "Portal31To34",
        "Portal32To11",
        "Portal32To31",
        "Portal33To31",
        "Portal34To12",
        "Portal34To31",
        "Portal41To42",
        "Portal41To43",
        "Portal41To44",
        "Portal42To12",
        "Portal42To41",
        "Portal43To41",
        "Portal44To13",
        "Portal44To41"
        ]
    for objectName in Portals:
        for obname in Blender.Library.Datablocks(dbname):
            if obname == objectName:
                Blender.Library.Load(obname,dbname,0)
    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

def import_linking_ped_objects(filename):
    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()
    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"
    objectPrefixes = ["bookc","panelc","cityCamRgn","cityCam","crc","pedc"]
    objects = { 1:[100,101],
        151:[102,103,104,105],
        152:[106,107,108,109],
        153:[110,111,112,113],
        251:[200,201,206,211],
        252:[202,203,204,205],
        253:[207,208,209,210],
        254:[212,213,214,215],
        351:[300,301,306,311],
        352:[302,303,304,305],
        353:[307,308,309,310],
        354:[312,313,314,315],
        451:[400,401,406,411],
        452:[402,403,404,405],
        453:[407,408,409,410],
        454:[412,413,414,415]
        }
    for pageNum in objects:
        print "Importing objects in page",pageNum
        for number in objects[pageNum]:
            for prefix in objectPrefixes:
                objectName = prefix+str(number)
                print "\t",objectName
                for obname in Blender.Library.Datablocks(dbname):
                    if obname == objectName:
                        Blender.Library.Load(obname,dbname,0)
                        obj = Blender.Object.Get(objectName)
                        try:
                            p=obj.getProperty("page_num")
                            obj.removeProperty(p)
                        except (AttributeError, RuntimeError):
                            pass
                        obj.addProperty("page_num",str(pageNum))
                        break
    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

def import_linking_points(filename):
    Blender.Library.Open(filename)
    groups = Blender.Library.LinkableGroups()
    dbname = 'Object'
    if not dbname in groups:
        raise RuntimeError, "Could not find object database in groups"
    for district in range(4):
        shellLimit = 16
        if district == 0:
            shellLimit = 14
        for shellNum in range(shellLimit):
            SHELL_STRING="%d%02d" %(district+1,shellNum)
            objectName = "lpc"+SHELL_STRING
            print "Importing",objectName
            for obname in Blender.Library.Datablocks(dbname):
                if obname == objectName:
                    Blender.Library.Load(obname,dbname,0)
                    obj = Blender.Object.Get(objectName)
                    try:
                        p=obj.getProperty("page_num")
                        obj.removeProperty(p)
                    except (AttributeError, RuntimeError):
                        pass
                    obj.addProperty("page_num","0")
                    break
    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()
 
def assign_shell_objects():
    SHELL_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Shell Number: ', SHELL_STRING, 19, 19, 'Number of shell to assign'),\
	]
		
    if not Blender.Draw.PupBlock('Specify shell number...', pup_block) or\
    not SHELL_STRING.val:
        return 0
		
    SHELL_STRING = SHELL_STRING.val
    shellNum = int(SHELL_STRING)

    #DEBUG
    print "Working on shell %d..." % (shellNum)
    assignCount = 0
    skipCount = 0
    obsel = Blender.Scene.GetCurrent().objects.selected

    for obj in obsel:
        skip = False
        # Rename it
        tokens = obj.name.split(".")
        if tokens[0].startswith("Wal") or\
        tokens[0].startswith("Top") or\
        tokens[0].startswith("Crn"):
            tokens[0] = tokens[0][0:3]
        elif tokens[0].startswith("Str") or\
        tokens[0].startswith("Flr"):
            tokens[0] = tokens[0][0:3]
            if len(tokens) > 1:
                tokens[0] += "COL"
        elif not tokens[0].startswith("ShCln") and\
        not tokens[0].startswith("LitShFrnt") and\
        not tokens[0].startswith("LitShSide") and\
        not tokens[0].startswith("LitShSd") and\
        not tokens[0].startswith("LitShFrnt") and\
        not tokens[0].startswith("FootStp"):
            print "Skipping object",obj.name
            skipCount += 1
            skip = True
        if not skip:
            obj.name = "%d%s" % (shellNum,tokens[0])

        # Set the page number
        try:
            p=obj.getProperty("page_num")
            obj.removeProperty(p)
        except (AttributeError, RuntimeError):
            pass
        obj.addProperty("page_num",SHELL_STRING)
        assignCount += 1

        # Delete the name property
        try:
            p=obj.getProperty("name")
            obj.removeProperty(p)
        except (AttributeError, RuntimeError):
            pass
            

    message = "Updated %d objects" % (assignCount)
    if (skipCount > 0):
        message += " and skipped %d objects" % (skipCount)
    message += "."
    Blender.Draw.PupMenu(message)


def ignore_textures():
    list = Blender.Object.Get()
    propsAdded = 0
    for obj in list:
        if obj.type == 'Mesh':
            type=None
            try:
                p = obj.getProperty("type")
                type=str(p.getData())
            except (AttributeError, RuntimeError):
                pass
            page_num = 0
            try:
                p = obj.getProperty("page_num")
                page_num=int(p.getData())
            except (AttributeError, RuntimeError):
                pass
            if type != "region" and type != "svconvex" and type != "collider" and page_num != 0:
               obj.addProperty("ignorePPT",True,'BOOL')
               propsAdded += 1
    Blender.Draw.PupMenu('Ignored textures in %d objects.' % (propsAdded))

def extension_remove():
    list = Blender.Object.Get()
    extensionsRemoved = 0
    for obj in list:
        if obj.type == 'Mesh':
            mesh = Blender.Mesh.Get(obj.data.name)
            type=None
            try:
                p = obj.getProperty("type")
                type=str(p.getData())
            except (AttributeError, RuntimeError):
                pass
            page_num = 0
            try:
                p = obj.getProperty("page_num")
                page_num=int(p.getData())
            except (AttributeError, RuntimeError):
                pass
            if type != "region" and type != "svconvex" and type != "collider" and page_num != 0 and len(mesh.materials) > 0:
               mat = mesh.materials[0]
               if mat != None:
                   for mtex in mat.getTextures():
                       if mtex != None and mtex.tex != None:
                           tokens = mtex.tex.name.split(".")
                           if len(tokens) > 1:
                               print "%s => %s" % (mtex.tex.name,tokens[0])
                               mtex.tex.name = tokens[0]
                               extensionsRemoved += 1
                           if mtex.tex.image != None:
                               tokens = mtex.tex.image.name.split(".")
                               if len(tokens) > 1:
                                   print "%s => %s" % (mtex.tex.image.name,tokens[0])
                                   mtex.tex.image.name = tokens[0]
                                   extensionsRemoved += 1
                           
    Blender.Draw.PupMenu('Removed extensions from texture names in %d objects.' % (extensionsRemoved))

def add_reserved_sign():
    SHELL_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Shell Number: ', SHELL_STRING, 19, 19, 'Which shell needs a sign?'),\
	]
		
    if not Blender.Draw.PupBlock('Specify shell number...', pup_block) or\
    not SHELL_STRING.val:
        return 0
		
    SHELL_STRING = SHELL_STRING.val
    shellNum = int(SHELL_STRING)

    # Get the stairs collider, in order to get the sign location and orientation
    objName = SHELL_STRING + "StrCOL"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    bb = obj.getBoundBox()
    center = [bb[0][n]+(bb[-2][n]-bb[0][n])/2.0 for n in [0,1,2]]
    RotZ = obj.RotZ + 3.14159


    # Deselect everything
    scn = Blender.Scene.GetCurrent()
    scn.objects.selected = []
    scn.objects.active = None

    # Get the sign and duplicate it
    objName = "ShReservedSign"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    scn.objects.selected = [obj]
    scn.objects.active = obj
    Blender.Object.Duplicate()
    obj = scn.objects.active
    obj.LocX = center[0]
    obj.LocY = center[1]
    obj.RotZ = RotZ
    obj.name = SHELL_STRING + objName

    # Set the page number
    try:
        p=obj.getProperty("page_num")
        obj.removeProperty(p)
    except (AttributeError, RuntimeError):
        pass
    obj.addProperty("page_num",SHELL_STRING)

    # Deselect everything
    scn = Blender.Scene.GetCurrent()
    scn.objects.selected = []
    scn.objects.active = None

    # Get the sign collider and duplicate it
    objName = "ShReservedSignCOL"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    scn.objects.selected = [obj]
    scn.objects.active = obj
    Blender.Object.Duplicate()
    obj = scn.objects.active
    obj.LocX = center[0]
    obj.LocY = center[1]
    obj.RotZ = RotZ
    obj.name = SHELL_STRING + objName

    # Set the page number
    try:
        p=obj.getProperty("page_num")
        obj.removeProperty(p)
    except (AttributeError, RuntimeError):
        pass
    obj.addProperty("page_num",SHELL_STRING)

    # Pin this object
    AlcScript.LoadFromBlender()
    objscript = AlcScript.objects.FindOrCreate(obj.name)
    StoreInDict(objscript,"physical.pinned","true")

    # Write out the alcscript to a text object, then delete the script
    blendtext = alcFindBlenderText(AlcScript.ObjectTextFileName)
    blendtext.clear()
    alctext = AlcScript.objects.Write()
    blendtext.write(alctext)
    AlcScript.objects = None
    AlcScript.book = None

    Blender.Draw.PupMenu('Created sign and collider for shell %s.' % (SHELL_STRING))

def add_shell_image_wrap():
    for district in range(4):
        shellLimit = 40
        if district == 0:
            shellLimit = 30
        for shellNum in range(shellLimit):
            SHELL_STRING="%d%02d" %(district+1,shellNum+1)
            print "Adding shell image to shell",SHELL_STRING,"..."
            add_shell_image(SHELL_STRING)

def add_shell_image(SHELL_STRING):
    # Get the stairs, in order to get the shell image location and orientation
    objName = SHELL_STRING + "Str"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    bb = obj.getBoundBox()
    center = [bb[0][n]+(bb[-2][n]-bb[0][n])/2.0 for n in [0,1,2]]
    RotZ = obj.RotZ + 3.14159


    # Deselect everything
    scn = Blender.Scene.GetCurrent()
    scn.objects.selected = []
    scn.objects.active = None

    # Get the shell image and duplicate it
    objName = "ShellImage"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    scn.objects.selected = [obj]
    scn.objects.active = obj
    Blender.Object.Duplicate()
    obj = scn.objects.active
    obj.LocX = center[0]
    obj.LocY = center[1]
    obj.RotZ = RotZ
    obj.name = SHELL_STRING + objName

    # Set the page number
    try:
        p=obj.getProperty("page_num")
        obj.removeProperty(p)
    except (AttributeError, RuntimeError):
        pass
    obj.addProperty("page_num",SHELL_STRING)

def add_col_wrap():
    for district in range(4):
        shellLimit = 16
        if district == 0:
            shellLimit = 14
        for shellNum in range(shellLimit):
            SHELL_STRING="%d%02d" %(district+1,shellNum)
            print "Adding collider to linkpoint",SHELL_STRING,"..."
            add_col(SHELL_STRING)

def add_col(SHELL_STRING):
    # Get the linkpoint, in order to get the collider location and orientation
    objName = "lpc" + SHELL_STRING
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    center = obj.getLocation("worldspace")
    RotZ = obj.RotZ + 3.14159


    # Deselect everything
    scn = Blender.Scene.GetCurrent()
    scn.objects.selected = []
    scn.objects.active = None

    # Get the shell image and duplicate it
    objName = "lpcCol"
    obj = Blender.Object.Get(objName)
    if obj == None:
        raise RuntimeError, "Could not locate object %s" % (objName)
    scn.objects.selected = [obj]
    scn.objects.active = obj
    Blender.Object.Duplicate()
    obj = scn.objects.active
    obj.LocX = center[0]
    obj.LocY = center[1]
    obj.LocZ = center[2]-3
    obj.RotZ = RotZ
    obj.name = "lpc" + SHELL_STRING + "Col"

    # Set the page number
    try:
        p=obj.getProperty("page_num")
        obj.removeProperty(p)
    except (AttributeError, RuntimeError):
        pass
    obj.addProperty("page_num","0")


def create_shell_file(name):
    Blender.Library.Open(name)
    SHELL_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Shell Number: ', SHELL_STRING, 19, 19, 'Which shell needs a sign?'),\
	]
		
    if not Blender.Draw.PupBlock('Specify shell number...', pup_block) or\
    not SHELL_STRING.val:
        return 0
		
    SHELL_STRING = SHELL_STRING.val
    shellNum = int(SHELL_STRING)


    groups = Blender.Library.LinkableGroups()

    # Get the shell scene objects
    dbname = 'Object'
    shellObjects = []
    if dbname in groups:
        for obname in Blender.Library.Datablocks(dbname):
            if obname.startswith(SHELL_STRING):
                Blender.Library.Load(obname,dbname,0)
                shellObjects.append(obname)

    # Get the object scripts
    dbname = 'Text'
    if dbname in groups:
        db = Blender.Library.Datablocks(dbname)
        if AlcScript.ObjectTextFileName in db:
            Blender.Library.Load(AlcScript.ObjectTextFileName,dbname,0)
        else:
            raise RuntimeException,"Could not find Text object %s" %(AlcScript.ObjectTextFileName)

    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

    # Create the book text object with just this page
    age = { \
        "sequenceprefix" : 189, \
        "startdatetime" : 0, \
        "daylength" : 24.0, \
        "maxcapacity" : 150, \
        "lingertime" : 180, \
        }
    text = "age:\n"
    for key in age.keys():
        text += "\t" + key +": " + str(age[key]) + "\n"
    text += "\n"
    text += "\tpages:\n"
    text += "\t\t- index: " + SHELL_STRING + "\n"
    text += "\t\t  name: Shell" + SHELL_STRING + "\n"
    blendtxt=alcFindBlenderText(AlcScript.BookTextFileName)
    blendtxt.clear()
    blendtxt.write(text)
    
    # Strip out all objects from the AlcScript except for this shell's objects
    AlcScript.LoadFromBlender()
    newContent = {}
    for obname in shellObjects:
        if obname in AlcScript.objects.content:
            newContent[obname] = AlcScript.objects.content[obname]
    AlcScript.objects.content = newContent
    text = AlcScript.objects.Write()
    blendtxt = alcFindBlenderText(AlcScript.ObjectTextFileName)
    blendtxt.clear()
    blendtxt.write(text)
    AlcScript.objects = None
    AlcScript.book = None
    
    # Save the file
    shellfile = Blender.sys.dirname(name) + "/Shell%s.blend" %(SHELL_STRING)
    print "Saving to",shellfile
    Blender.Save(shellfile,1)
    Blender.Draw.PupMenu('Created shell file %s.' % (shellfile))

def import_shell(name):
    Blender.Library.Open(name)
    SHELL_STRING = Blender.Draw.Create('')
		
    pup_block = [\
        ('Shell Number: ', SHELL_STRING, 19, 19, 'Which shell needs a sign?'),\
	]
		
    if not Blender.Draw.PupBlock('Specify shell number...', pup_block) or\
    not SHELL_STRING.val:
        return 0
		
    SHELL_STRING = SHELL_STRING.val
    shellNum = int(SHELL_STRING)


    groups = Blender.Library.LinkableGroups()

    # Get the shell scene objects
    dbname = 'Object'
    shellObjects = []
    if dbname in groups:
        for obname in Blender.Library.Datablocks(dbname):
            if obname.startswith(SHELL_STRING):
                Blender.Library.Load(obname,dbname,0)
                shellObjects.append(obname)

    # Get the object scripts
    dbname = 'Text'
    if dbname in groups:
        db = Blender.Library.Datablocks(dbname)
        if AlcScript.ObjectTextFileName in db:
            Blender.Library.Load(AlcScript.ObjectTextFileName,dbname,0)
        else:
            raise RuntimeException,"Could not find Text object %s" %(AlcScript.ObjectTextFileName)

    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()

    # Create the book text object with just this page
    age = { \
        "sequenceprefix" : 189, \
        "startdatetime" : 0, \
        "daylength" : 24.0, \
        "maxcapacity" : 150, \
        "lingertime" : 180, \
        }
    text = "age:\n"
    for key in age.keys():
        text += "\t" + key +": " + str(age[key]) + "\n"
    text += "\n"
    text += "\tpages:\n"
    text += "\t\t- index: " + SHELL_STRING + "\n"
    text += "\t\t  name: Shell" + SHELL_STRING + "\n"
    blendtxt=alcFindBlenderText(AlcScript.BookTextFileName)
    blendtxt.clear()
    blendtxt.write(text)
    
    # Strip out all objects from the AlcScript except for this shell's objects
    AlcScript.LoadFromBlender()
    newContent = {}
    for obname in shellObjects:
        if obname in AlcScript.objects.content:
            newContent[obname] = AlcScript.objects.content[obname]
    AlcScript.objects.content = newContent
    text = AlcScript.objects.Write()
    blendtxt = alcFindBlenderText(AlcScript.ObjectTextFileName)
    blendtxt.clear()
    blendtxt.write(text)
    AlcScript.objects = None
    AlcScript.book = None
    
    # Save the file
    shellfile = Blender.sys.dirname(name) + "/Shell%s.blend" %(SHELL_STRING)
    print "Saving to",shellfile
    Blender.Save(shellfile,1)
    Blender.Draw.PupMenu('Created shell file %s.' % (shellfile))


def importWalls(name):
    Blender.Library.Open(name)
    groups = Blender.Library.LinkableGroups()

    # Get the Wall objects
    dbname = 'Object'
    shellObjects = []
    if dbname in groups:
        for obname in Blender.Library.Datablocks(dbname):
            if obname.startswith('DstWal.'):
                Blender.Library.Load(obname,dbname,0)
                shellObjects.append(obname)

    # Close the library
    Blender.Library.Update()
    Blender.Library.Close()


def do_main():
    args = __script__['arg']
    w = args.split("_")
    if w[1]=="assign":
        assign_shell_objects()
    elif w[1]=="ignore":
        ignore_textures()
    elif w[1]=="extrem":
        extension_remove()
    elif w[1]=="reserved":
        add_reserved_sign()
    elif w[1]=="addimage":
        add_shell_image_wrap()
    elif w[1]=="deleteobj":
        delete_objects()
    elif w[1]=="create":
        filename = './Shells.blend'
        if os.path.exists(filename):
            create_shell_file(filename)
        else:
            print "could not find file", filename
            Blender.Window.FileSelector(create_shell_file,"Locate the Shells.blend file")
    elif w[1]=="importwalls":
        filename = './City0.8.8.blend'
        if os.path.exists(filename):
            importWalls(filename)
        else:
            print "could not find file", filename
            Blender.Window.FileSelector(importWalls,"Locate the City0.8.8.blend file")
    elif w[1]=="importsubdistrict":
        filename = './City0.8.8.blend'
        if os.path.exists(filename):
            import_subdistrict_objects2(filename)
        else:
            print "could not find file", filename
            Blender.Window.FileSelector(import_subdistrict_objects2,"Locate the City0.8.8.blend file")
    elif w[1]=="importshellimg":
        Blender.Window.FileSelector(import_shell_image_wrap,"Locate the .blend file")
    elif w[1]=="uncompress":
        uncompress_texture()
    elif w[1]=="addcol":
        add_col_wrap()
    elif w[1]=="importportal":
        Blender.Window.FileSelector(import_portals,"Locate the .blend file")
    elif w[1]=="importlinkped":
        Blender.Window.FileSelector(import_linking_ped_objects,"Locate the .blend file")
    elif w[1]=="importlp":
        Blender.Window.FileSelector(import_linking_points,"Locate the .blend file")
    else:
        raise "Unknown options %s" %(w)


#Main code
do_main()

